#define _CRT_SECURE_NO_WARNINGS
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "DMPro_Types.h"

#include "LoadBmp.cpp"

const int   // the images maximum dimensions
  maxrowQ = 5000,
  maxcolQ = 8192;

void Decode(const char* image, PDM_Decoder pDecoder, TDM_OptMode opt);

TRow*    pbits;

int main(int argc, char **argv)
{   
   TRow     pmembits;  // Image in Memory
   char endrun;
	int res;

	PDM_Decoder pDecoder = NULL;
   TDM_OptMode opt;

   pmembits = (TRow ) malloc(maxrowQ * maxcolQ);      //  Image in Memory
   pbits    = (TRow*) malloc(maxrowQ * sizeof(TRow)); // pointers to ScanLines
   for (int row = 0; row < maxrowQ; row++){
     pbits[row] = &pmembits[maxcolQ*row];
   }
   
	pDecoder = Connect_DM_Decoder(maxcolQ,maxrowQ);
	if (pDecoder == NULL)
	{
		printf("Failed Connect_DM_Decoder\n");
		goto doExit;
	}
   // Set options
	opt.maxDMCount = 1;
   opt.cellColor  = CL_ANY; // equal to default value
   opt.mirrorMode = MM_ANY;
   opt.speedMode  = DMSP_ULTIMATEPLUS;  
   opt.qualityMask = DM_QM_ALL;
   opt.labelMode  = LM_ST_DOT;
   opt.timeOut    = 0;
   opt.filterMode = FM_NON;
   opt.qzMode     = DMQZ_NORMAL; 
	
	// Set ProductID
   strcpy(opt.productID, "s#123456789");
	// Add userId
	if ((res = AddUserID(pDecoder, "*User_Company_Name*133511489C993057B47F82*D6522625657B0E1E656870")) != 0)
	{
		printf("Failed AddUserID: %d\n", res);
		goto doExit;
	}

	// Decoding unprotected data matrix
	opt.signMode   = DM_SM_NO;
	Decode("NotProt.bmp", pDecoder, opt);

	// Decoding data matrix with userId autentification
	opt.signMode   = DM_SM_USER;
	Decode("USig.bmp", pDecoder, opt);

	// Decoding data matrix with encription
	opt.signMode   = DM_SM_PRODUCT;
	Decode("PSig.bmp", pDecoder, opt);

	// Decoding data matrix with encription and userId autentification
	opt.signMode   = DM_SM_ALL;
	Decode("USigPSig.bmp", pDecoder, opt);

	
doExit:
	if(pDecoder) 
		Disconnect_DM_Decoder(pDecoder);

   free(pbits);
   free(pmembits);

	printf("\nPush 'Enter' to exit>");
   scanf("%c", &endrun);
   return 0;
}

void Decode(const char* image, PDM_Decoder pDecoder, TDM_OptMode opt)
{
	TDM_ImageInfo* pdm_imageinfo;
   TDM_Info* pdm_info;
	char Symbology_Identifier[4];

	// Create options
	PDM_Options dec1 = Create_DM_Options(pDecoder, opt);
	if (dec1 == NULL)
	{
	 printf("Failed Create_DM_Options\n");
	 return;
	}
	printf("%s ", image);

	int rowcount = maxrowQ;
	int colcount = maxcolQ;
	int res = LoadBMP((char*)image, pbits, rowcount, colcount);
	if (res == 0)
	{
	  res = DecodeDM_Bits(dec1, rowcount, colcount, pbits);
	  pdm_imageinfo = GetDM_ImageInfo(dec1);
	  printf("\n DM count = %4d", pdm_imageinfo->DMCount);
	}
	else
	{
	  printf("Failed LoadBMP. %s\n\n", image);
	  return;
	}

	if (pdm_imageinfo->RejectionReason == DM_RR_OK) // Decode OK
	{
	  for (int i = 0; i < pdm_imageinfo->DMCount; i++)
	  {
		  pdm_info = GetDM_Info(dec1, i);
		  printf("\n rowcols: ");
		  for (int k = 0; k < 4; k++)
			  printf("(%4d,%4d),   ", int(pdm_info->rowcols[2 * k]), int(pdm_info->rowcols[2 * k + 1]));

		  printf("\n\n Authentication ");
		  if (pdm_info->UserSign == DM_AU_OK)
			  printf("OK");
		  else if (pdm_info->UserSign == DM_AU_FAILURE)
			  printf("Failed");
		  else
			  printf("N/A");

		  printf("\n Decryption ");
		  if (pdm_info->ProductSign == DM_AU_OK)
			  printf("OK");
		  else if (pdm_info->ProductSign == DM_AU_FAILURE)
			  printf("Failed");
		  else
			  printf("N/A");

		  printf("\n");
		  if (pdm_info->pchlen > 0)
		  {
			  printf("\n Decoded array: %s", pdm_info->pch);
			  strncpy(Symbology_Identifier, (char*)&(pdm_info->pch[-3]), 3);
			  Symbology_Identifier[3] = 0;
			  printf("\n Symbology_Id = %s,  RSErr = %4d", Symbology_Identifier, pdm_info->RSErr);
		  }
		  printf("\n Dimensions: %4d * %4d", pdm_info->VDim, pdm_info->HDim);
		  printf("\n axial_nonuniformity (grade)  = %7.2f  (%d)", pdm_info->quality.axial_nonuniformity, int(pdm_info->quality.axial_nonuniformity_grade));
		  printf("\n grid_nonuniformity  (grade)  = %7.2f  (%d)", pdm_info->quality.grid_nonuniformity, int(pdm_info->quality.grid_nonuniformity_grade));
		  printf("\n symbol_contrast     (grade)  = %7.2f  (%d)", pdm_info->quality.symbol_contrast, int(pdm_info->quality.symbol_contrast_grade));
		  printf("\n unused_error_corr.  (grade)  = %7.2f  (%d)", pdm_info->quality.unused_error_correction, int(pdm_info->quality.unused_error_correction_grade));
		  printf("\n vert. & horiz. print_growth  = %7.2f,%7.2f", pdm_info->quality.vertical_print_growth, pdm_info->quality.horizontal_print_growth);
		  printf("\n fixed_patt_damage.  (grade)  = %7.2f  (%d)", pdm_info->quality.fixed_pattern_damage, int(pdm_info->quality.fixed_pattern_damage_grade));
		  printf("\n (modulation grade)           =          (%d)", int(pdm_info->quality.modulation_grade));
		  printf("\n (decode grade)               =          (%d)", int(pdm_info->quality.decode_grade));
		  printf("\n (overall grade)              =          (%d)", int(pdm_info->quality.overall_grade)); 
	  } 
	}
	printf("\n---------------------------------------------------\n\n");

	// Delete options
	Delete_DM_Options(dec1);
}
